/**
 * PrintInfo.cpp
 *
 * This file is part of the LibTiePie programming examples.
 *
 * Find more information on http://www.tiepie.com/LibTiePie .
 */

#include "PrintInfo.h"
#include <iomanip>
#include <iostream>
#include <sstream>

using namespace std;
using namespace LibTiePie;

const string GeneratorModes[GMN_COUNT] = { "Continuous" , "Burst count" , "Gated periods" , "Gated" , "Gated period start" , "Gated period finish" , "Gated run" , "Gated run output" , "Burst sample count" , "Burst sample count output" , "Burst segment count" , "Burst segment count output" };
const string ClockOutputTypes[CON_COUNT] = { "Disabled" , "Sample" , "Fixed" };
const string ClockSources[CSN_COUNT] = { "External" , "Internal" };
const string ConnectorTypes[CONNECTORTYPE_COUNT] = { "BNC" , "Banana" , "Power plug" };
const string Couplings[CKN_COUNT] = { "DCV" , "ACV" , "DCA" , "ACA"  , "Ohm" };
const string DeviceTypes[DEVICETYPE_COUNT] = { "Oscilloscope" , "Generator" , "I2CHost" };
const string FrequencyModes [FMN_COUNT] = { "Signal frequency" , "Sample frequency" };
const string MeasureModes[MMN_COUNT] = { "Stream" , "Block" };
const string AutoResolutionModes[ARN_COUNT] = { "Disabled" , "Native only" , "All" };
const string SignalTypes[STN_COUNT] = { "Sine" , "Triangle" , "Square" , "DC" , "Noise", "Arbitrary" , "Pulse" };
const string TriggerConditions[TCN_COUNT] = { "None" , "Smaller" , "Larger" , "Inside" , "Outside" };
const string TriggerKinds[TKN_COUNT] = { "Rising edge" , "Falling edge" , "In window" , "Out Window" , "Any edge" , "Enter window" , "Exit window" , "Pulse width positive" , "Pulse width negative" };
const string TriggerLevelModes[TLMN_COUNT] = { "Relative" , "Absolute" };
const string TriggerOutputEvents[TOEN_COUNT] = { "Generator start" , "Generator stop" , "Generator new period" };

void printLibraryInfo()
{
  try
  {
    cout << "Library:" << endl;
    cout << "  Version: " << convertVersion(Library::version()) << Library::versionExtra() << endl;
    cout << "  Configuration: " << hex << uppercase << "0x";
    const vector<uint8_t> Configuration(Library::config());
    for(vector<uint8_t>::size_type i = 0; i < Configuration.size(); i++)
      cout << setw(2) << setfill('0') << (unsigned int)Configuration[i];
    cout << dec <<  endl;
  }
  catch(const exception& e)
  {
    cerr << "Exception: " << e.what() << endl;
  }
}

void printDeviceInfo(const Device* device)
{
  if(!device)
  {
    cerr << "Null pointer in printDeviceInfo()" << endl;
    return;
  }

  try
  {
    cout << "Device:" << endl;
    cout << "  Name                      : " << device->name() << endl;
    cout << "  Short name                : " << device->nameShort() << endl;
    cout << "  Serial number             : " << device->serialNumber() << endl;
    try
    {
      cout << "  Calibration date          : " << convertDate(device->calibrationDate()) << endl;
    }
    catch(const exception&)
    {
    }
    cout << "  Product id                : " << device->productId() << endl;
    cout << "  Vendor id                 : " << device->vendorId() << endl;
    try
    {
      cout << "  Driver version            : " << convertVersion(device->driverVersion()) << endl;
    }
    catch(const exception&)
    {
    }
    try
    {
      cout << "  Firmware version          : " << convertVersion(device->firmwareVersion()) << endl;
    }
    catch(const exception&)
    {
    }

    const Oscilloscope* scp = dynamic_cast<const Oscilloscope*>(device);
    if(scp)
      printOscilloscopeInfo(scp);

    const Generator* gen = dynamic_cast<const Generator*>(device);
    if(gen)
      printGeneratorInfo(gen);

    const I2CHost* i2c = dynamic_cast<const I2CHost*>(device);
    if(i2c)
      printI2CInfo(i2c);

  }
  catch(const exception& e)
  {
    cerr << "Exception: " << e.what() << endl;
  }
}

void printOscilloscopeInfo(const Oscilloscope* scp)
{
  if(!scp)
  {
    cerr << "Null pointer in printOscilloscopeInfo()" << endl;
    return;
  }

  try
  {
    cout << "Oscilloscope:" << endl;
    cout << "  Channel count             : " << scp->channels.count() << endl;
    cout << "  Connection test           : " << boolToStr(scp->hasConnectionTest()) << endl;
    cout << "  Measure modes             : " << measureModeToStr(scp->measureModes()) << endl;
    cout << "  Measure mode              : " << measureModeToStr(scp->measureMode()) << endl;
    cout << "  Auto resolution modes     : " << autoResolutionModeToStr(scp->autoResolutionModes()) << endl;
    cout << "  Auto resolution mode      : " << autoResolutionModeToStr(scp->autoResolutionMode()) << endl;

    cout << "  Resolutions               : ";
    const vector<uint8_t> Resolutions(scp->resolutions());
    for(vector<uint8_t>::size_type i = 0; i < Resolutions.size(); i++)
    {
      if(i != 0)
        cout << ", ";

      cout << (unsigned int)Resolutions[i];
    }
    cout << endl;

    cout << "  Resolution                : " << (unsigned int)scp->resolution() << endl;
    cout << "  Resolution enhanced       : " << boolToStr(scp->isResolutionEnhanced()) << endl;
    cout << "  Clock outputs             : " << clockOutputToStr(scp->clockOutputs()) << endl;
    cout << "  Clock output              : " << clockOutputToStr(scp->clockOutput()) << endl;

    try
    {
      const vector<double> ClockOutputFrequencies(scp->clockOutputFrequencies());
      cout << "  Clock output frequencies  : ";
      for (vector<double>::size_type i = 0; i < ClockOutputFrequencies.size(); i++)
      {
        if (i != 0)
          cout << ", ";

        cout << ClockOutputFrequencies[i];
      }
      cout << endl;
      cout << "  Clock output frequency    : " << scp->clockOutputFrequency() << endl;
    }
    catch (const exception&)
    {
    }

    cout << "  Clock sources             : " << clockSourceToStr(scp->clockSources()) << endl;
    cout << "  Clock source              : " << clockSourceToStr(scp->clockSource()) << endl;

    try
    {
      const vector<double> ClockSourceFrequencies(scp->clockSourceFrequencies());
      cout << "  Clock source frequencies  : ";
      for(vector<double>::size_type i = 0; i < ClockSourceFrequencies.size(); i++)
      {
        if(i != 0)
          cout << ", ";

        cout << ClockSourceFrequencies[i];
      }
      cout << endl;
      cout << "  Clock source frequency      : " << scp->clockSourceFrequency() << endl;
    }
    catch (const exception&)
    {
    }

    cout << "  Record length max         : " << scp->recordLengthMax() << endl;
    cout << "  Record length             : " << scp->recordLength() << endl;
    cout << "  Sample frequency max      : " << scp->sampleFrequencyMax() << endl;
    cout << "  Sample frequency          : " << scp->sampleFrequency() << endl;

    if(scp->measureMode() == MM_BLOCK)
    {
      cout << "  Segment count max         : " << scp->segmentCountMax() << endl;
      if(scp->segmentCountMax() > 1)
        cout << "  Segment count             : " << scp->segmentCount() << endl;
    }

    if(scp->hasTrigger())
    {
      cout << "  Pre sample ratio          : " << scp->preSampleRatio() << endl;

      cout << "  Trigger time out          : ";
      if(scp->triggerTimeOut() == TO_INFINITY)
        cout << "Infinite" ;
      else
        cout << scp->triggerTimeOut();;
      cout << endl;

      if(scp->hasTriggerDelay())
      {
        cout << "  Trigger delay max         : " << scp->triggerDelayMax() << endl;
        cout << "  Trigger delay             : " << scp->triggerDelay() << endl;
      }

      if(scp->hasTriggerHoldOff())
      {
        cout << "  Trigger hold off count max: " << scp->triggerHoldOffCountMax() << endl;
        cout << "  Trigger hold off count    : " << scp->triggerHoldOffCount() << endl;
      }
    }

    const uint16_t channelCount = scp->channels.count();
    for(uint16_t ch = 0; ch < channelCount; ch++)
    {
      const OscilloscopeChannel& channel = scp->channels[ch];
      cout << "  Ch" << (ch + 1) << ":" << endl;
      cout << "    Connector type          : " << connectorTypeToStr(channel.connectorType()) << endl;
      cout << "    Differential            : " << boolToStr(channel.isDifferential()) << endl;
      cout << "    Impedance               : " << channel.impedance() << endl;
      cout << "    Connection test         : " << boolToStr(channel.hasConnectionTest()) << endl;
      cout << "    Available               : " << boolToStr(channel.isAvailable()) << endl;
      cout << "    Enabled                 : " << boolToStr(channel.enabled()) << endl;

      cout << "    Bandwidths              : ";
      const vector<double> Bandwidths(channel.bandwidths());
      for(vector<double>::size_type i = 0; i < Bandwidths.size(); i++)
      {
        if(i != 0)
          cout << ", ";

        cout << Bandwidths[i];
      }
      cout << endl;
      cout << "    Bandwidth               : " << channel.bandwidth() << endl;

      cout << "    Couplings               : " << couplingToStr(channel.couplings()) << endl;
      cout << "    Coupling                : " << couplingToStr(channel.coupling()) << endl;
      cout << "    Auto ranging            : " << boolToStr(channel.autoRanging()) << endl;

      cout << "    Ranges                  : ";
      const vector<double> Ranges(channel.ranges());
      for(vector<double>::size_type i = 0; i < Ranges.size(); i++)
      {
        if(i != 0)
          cout << ", ";

        cout << Ranges[i];
      }
      cout << endl;

      cout << "    Range                   : " << channel.range() << endl;
      cout << "    Probe gain              : " << channel.probeGain() << endl;
      cout << "    Probe offset            : " << channel.probeOffset() << endl;
      if(channel.hasSafeGround())
      {
        cout << "    SafeGround enabled      : " << boolToStr(channel.safeGroundEnabled()) << endl;
        cout << "    SafeGround threshold max: " << channel.safeGroundThresholdMax() << endl;
        cout << "    SafeGround threshold min: " << channel.safeGroundThresholdMin() << endl;
        cout << "    SafeGround threshold    : " << channel.safeGroundThreshold() << endl;
      }

      if(channel.hasTrigger())
      {
        cout << "    Trigger:" << endl;
        cout << "      Available             : " << boolToStr(channel.trigger->isAvailable()) << endl;
        cout << "      Enabled               : " << boolToStr(channel.trigger->enabled()) << endl;
        cout << "      Kinds                 : " << triggerKindToStr(channel.trigger->kinds()) << endl;
        cout << "      Kind                  : " << triggerKindToStr(channel.trigger->kind()) << endl;
        cout << "      Level modes           : " << triggerLevelModeToStr(channel.trigger->levelModes())<< endl;
        cout << "      Level mode            : " << triggerLevelModeToStr(channel.trigger->levelMode())<< endl;

        const uint32_t triggerLevelCount = channel.trigger->Levels.count();
        cout << "      Levels                : ";
        for(uint32_t i = 0; i < triggerLevelCount; i++)
        {
          if(i != 0)
            cout << ", ";

          cout << channel.trigger->Levels[i];
        }
        cout << endl;

        const uint32_t triggerHysteresisCount = channel.trigger->Hystereses.count();
        cout << "      Hystereses            : ";
        for(uint32_t i = 0; i < triggerHysteresisCount; i++)
        {
          if(i != 0)
            cout << ", ";

          cout << channel.trigger->Hystereses[i];
        }
        cout << endl;

        uint32_t triggerConditions = channel.trigger->conditions();
        cout << "      Conditions            : " << triggerConditionToStr(triggerConditions) << endl;
        if(triggerConditions != TCM_NONE)
          cout << "      Condition             : " << triggerConditionToStr(channel.trigger->condition()) << endl;

        const uint32_t triggerTimeCount = channel.trigger->Times.count();
        if(triggerTimeCount > 0)
        {
          cout << "      Times                 : ";
          for(uint32_t i = 0; i < triggerTimeCount; i++)
          {
            if(i != 0)
              cout << ", ";

            cout << channel.trigger->Times[i];
          }
          cout << endl;
        }
      }
    }

    printTriggerInputsInfo(scp->triggerInputs);
    printTriggerOutputsInfo(scp->triggerOutputs);
  }
  catch(const exception& e)
  {
    cerr << "Exception: " << e.what() << endl;
  }
}

void printGeneratorInfo(const Generator* gen)
{
  if(!gen)
  {
    cerr << "Null pointer in printGeneratorInfo()" << endl;
    return;
  }

  try
  {
    cout << "Generator:" << endl;
    cout << "  Connector type            : " << connectorTypeToStr(gen->connectorType()) << endl;
    cout << "  Differential              : " << boolToStr(gen->isDifferential()) << endl;
    cout << "  Controllable              : " << boolToStr(gen->isControllable()) << endl;
    cout << "  Impedance                 : " << gen->impedance() << endl;
    cout << "  Resolution                : " << (unsigned int)gen->resolution() << endl;
    cout << "  Output value min          : " << gen->outputValueMin() << endl;
    cout << "  Output value max          : " << gen->outputValueMax() << endl;
    cout << "  Output on                 : " << boolToStr(gen->outputOn()) << endl;
    if(gen->hasOutputInvert())
    {
      cout << "  Output invert             : " << boolToStr(gen->outputInvert()) << endl;
    }

    cout << "  Modes native              : " << generatorModeToStr(gen->modesNative()) << endl;
    cout << "  Modes                     : " << generatorModeToStr(gen->modes()) << endl;
    if(gen->modes() != GMM_NONE)
    {
      cout << "  Mode                      : " << generatorModeToStr(gen->mode()) << endl;
      if(gen->mode() & GMM_BURST_COUNT)
      {
        cout << "  Burst active              : " << boolToStr(gen->isBurstActive()) << endl;
        cout << "  Burst count max           : " << gen->burstCountMax() << endl;
        cout << "  Burst count               : " << gen->burstCount() << endl;
      }
      if(gen->mode() & GMM_BURST_SAMPLE_COUNT)
      {
        cout << "  Burst sample count max    : " << gen->burstSampleCountMax() << endl;
        cout << "  Burst sample count        : " << gen->burstSampleCount() << endl;
      }
      if(gen->mode() & GMM_BURST_SEGMENT_COUNT)
      {
        cout << "  Burst segment count max   : " << gen->burstSegmentCountMax() << endl;
        cout << "  Burst segment count       : " << gen->burstSegmentCount() << endl;
      }
    }

    cout << "  Signal types              : " << signalTypeToStr(gen->signalTypes()) << endl;
    cout << "  Signal type               : " << signalTypeToStr(gen->signalType()) << endl;

    if(gen->hasAmplitude())
    {
      cout << "  Amplitude min             : " << gen->amplitudeMin() << endl;
      cout << "  Amplitude max             : " << gen->amplitudeMax() << endl;
      cout << "  Amplitude                 : " << gen->amplitude() << endl;

      cout << "  Amplitude ranges          : ";
      const vector<double> AmplitudeRanges(gen->amplitudeRanges());
      for(vector<double>::size_type i = 0; i < AmplitudeRanges.size(); i++)
      {
        if(i != 0)
          cout << ", ";

        cout << AmplitudeRanges[i];
      }
      cout << endl;

      cout << "  Amplitude range           : " << gen->amplitudeRange() << endl;
      cout << "  Amplitude auto ranging    : " << boolToStr(gen->amplitudeAutoRanging()) << endl;
    }

    if(gen->hasFrequency())
    {
      cout << "  Frequency modes           : " << frequencyModeToStr(gen->frequencyModes()) << endl;
      cout << "  Frequency mode            : " << frequencyModeToStr(gen->frequencyMode()) << endl;
      cout << "  Frequency min             : " << gen->frequencyMin() << endl;
      cout << "  Frequency max             : " << gen->frequencyMax() << endl;
      cout << "  Frequency                 : " << gen->frequency() << endl;
    }

    if(gen->hasOffset())
    {
      cout << "  Offset min                : " << gen->offsetMin() << endl;
      cout << "  Offset max                : " << gen->offsetMax() << endl;
      cout << "  Offset                    : " << gen->offset() << endl;
    }

    if(gen->hasPhase())
    {
      cout << "  Phase min                 : " << gen->phaseMin() << endl;
      cout << "  Phase max                 : " << gen->phaseMax() << endl;
      cout << "  Phase                     : " << gen->phase() << endl;
    }

    if(gen->hasSymmetry())
    {
      cout << "  Symmetry min              : " << gen->symmetryMin() << endl;
      cout << "  Symmetry max              : " << gen->symmetryMax() << endl;
      cout << "  Symmetry                  : " << gen->symmetry() << endl;
    }

    if(gen->hasWidth())
    {
      cout << "  Width min                 : " << gen->widthMin() << endl;
      cout << "  Width max                 : " << gen->widthMax() << endl;
      cout << "  Width                     : " << gen->width() << endl;
    }

    if(gen->hasEdgeTime())
    {
      cout << "  Leading edge time min     : " << gen->leadingEdgeTimeMin() << endl;
      cout << "  Leading edge time max     : " << gen->leadingEdgeTimeMax() << endl;
      cout << "  Leading edge time         : " << gen->leadingEdgeTime() << endl;
      cout << "  Trailing edge time min    : " << gen->trailingEdgeTimeMin() << endl;
      cout << "  Trailing edge time max    : " << gen->trailingEdgeTimeMax() << endl;
      cout << "  Trailing edge time        : " << gen->trailingEdgeTime() << endl;
    }

    if(gen->hasData())
    {
      cout << "  DataLength min            : " << gen->dataLengthMin() << endl;
      cout << "  DataLength max            : " << gen->dataLengthMax() << endl;
      cout << "  DataLength                : " << gen->dataLength() << endl;
    }

    printTriggerInputsInfo(gen->triggerInputs);
    printTriggerOutputsInfo(gen->triggerOutputs);
  }
  catch(const exception& e)
  {
    cerr << "Exception: " << e.what() << endl;
  }
}

void printI2CInfo(const I2CHost* i2c)
{
  if(!i2c)
  {
    cerr << "Null pointer in printI2CInfo()";
    return;
  }

  try
  {
    cout << "I2C Host:" << endl;

    cout << "  Internal addresses        : ";
    const vector<uint16_t> InternalAddresses(i2c->internalAddresses());
    for(vector<uint16_t>::size_type i = 0; i < InternalAddresses.size(); i++)
    {
      if(i != 0)
        cout << ", ";

      cout << InternalAddresses[i];
    }
    cout << endl;

    cout << "  Speed max                 : " << i2c->speedMax() << endl;
    cout << "  Speed                     : " << i2c->speed() << endl;

    printTriggerInputsInfo(i2c->triggerInputs);
    printTriggerOutputsInfo(i2c->triggerOutputs);
  }
  catch(const exception& e)
  {
    cerr << "Exception: " << e.what() << endl;
  }
}

void printTriggerInputsInfo(const TriggerInputs& TriggerInputs)
{
  const TriggerInputs::size_type Count = TriggerInputs.count();

  for(TriggerInputs::size_type i = 0; i < Count; i++)
  {
    const TriggerInput* triggerInput = &TriggerInputs[i];

    cout << "  TriggerInput " << i << ":" << endl;
    cout << "    Id                      : " << triggerInput->id() << endl;
    cout << "    Name                    : " << triggerInput->name() << endl;
    cout << "    Available               : " << boolToStr(triggerInput->isAvailable()) << endl;

    if(triggerInput->isAvailable())
    {
      cout << "    Enabled                 : " << boolToStr(triggerInput->enabled()) << endl;
      cout << "    Kinds                   : " << triggerKindToStr(triggerInput->kinds()) << endl;
      if(triggerInput->kinds() != TKM_NONE)
        cout << "    Kind                    : " << triggerKindToStr(triggerInput->kind()) << endl;
    }
  }
}

void printTriggerOutputsInfo(const TriggerOutputs& TriggerOutputs)
{
  const TriggerOutputs::size_type Count = TriggerOutputs.count();

  for(TriggerOutputs::size_type i = 0; i < Count; i++)
  {
    const TriggerOutput* triggerOutput = &TriggerOutputs[i];

    cout << "  TriggerOutput " << i << ":" << endl;
    cout << "    Id                      : " << triggerOutput->id() << endl;
    cout << "    Name                    : " << triggerOutput->name() << endl;
    cout << "    Enabled                 : " << boolToStr(triggerOutput->enabled()) << endl;
    cout << "    Events                  : " << triggerOutputEventToStr(triggerOutput->events()) << endl;
    cout << "    Event                   : " << triggerOutputEventToStr(triggerOutput->event()) << endl;
  }
}

// String conversion functions:
string generatorModeToStr(const uint64_t generatorModes)
{
  if(generatorModes == GM_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < GMN_COUNT; i++)
  {
    if(generatorModes & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(GeneratorModes[i]);
    }
  }

  return s;
}

string clockOutputToStr(const uint32_t clockOutputs)
{
  string s;

  for(unsigned int i = 0; i < CON_COUNT; i++)
  {
    if(clockOutputs & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(ClockOutputTypes[i]);
    }
  }

  return s;
}

string clockSourceToStr(const uint32_t clockSources)
{
  string s;

  for(unsigned int i = 0; i < CSN_COUNT; i++)
  {
    if(clockSources & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(ClockSources[i]);
    }
  }

  return s;
}

string connectorTypeToStr(const uint32_t connectorTypes)
{
  if(connectorTypes == CONNECTORTYPE_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < CONNECTORTYPE_COUNT; i++)
  {
    if(connectorTypes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(ConnectorTypes[i]);
    }
  }

  return s;
}

string convertDate(const TpDate_t date)
{
  if(date == 0)
    return "Unavailable";

  stringstream ss;

  ss << TPDATE_YEAR(date) << "-" << TPDATE_MONTH(date) << "-" << TPDATE_DAY(date);

  return ss.str();
}

string convertVersion(const TpVersion_t version)
{
  stringstream ss;

  ss << TPVERSION_MAJOR(version) << "." << TPVERSION_MINOR(version) << "." << TPVERSION_RELEASE(version) << "." << TPVERSION_BUILD(version);

  return ss.str();
}

string couplingToStr(const uint64_t couplings)
{
  if(couplings == CK_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < CKN_COUNT; i++)
  {
    if(couplings & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(Couplings[i]);
    }
  }

  return s;
}

string deviceTypeToStr(const uint32_t deviceTypes)
{
  string s;

  for(unsigned int i = 0; i < DEVICETYPE_COUNT; i++)
  {
    if(deviceTypes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(DeviceTypes[i]);
    }
  }

  return s;
}

string frequencyModeToStr(const uint32_t frequencyModes)
{
  if(frequencyModes == FM_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < FMN_COUNT; i++)
  {
    if(frequencyModes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(FrequencyModes[i]);
    }
  }

  return s;
}

string measureModeToStr(const uint32_t measureModes)
{
  if(measureModes == MM_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < MMN_COUNT; i++)
  {
    if(measureModes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(MeasureModes[i]);
    }
  }

  return s;
}

string autoResolutionModeToStr(const uint32_t autoResolutions)
{
  if(autoResolutions == AR_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < ARN_COUNT; i++)
  {
    if(autoResolutions & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(AutoResolutionModes[i]);
    }
  }

  return s;
}

string signalTypeToStr(const uint32_t signalTypes)
{
  if(signalTypes == ST_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < STN_COUNT; i++)
  {
    if(signalTypes & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(SignalTypes[i]);
    }
  }

  return s;
}

string triggerConditionToStr(const uint32_t triggerConditions)
{
  if(triggerConditions == TC_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < TCN_COUNT; i++)
  {
    if(triggerConditions & (1UL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(TriggerConditions[i]);
    }
  }

  return s;
}

string triggerKindToStr(const uint64_t triggerKinds)
{
  if(triggerKinds == TK_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < TKN_COUNT; i++)
  {
    if(triggerKinds & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(TriggerKinds[i]);
    }
  }

  return s;
}

string triggerLevelModeToStr(const uint32_t triggerLevelModes)
{
  if(triggerLevelModes == TLM_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < TLMN_COUNT; i++)
  {
    if(triggerLevelModes & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(TriggerLevelModes[i]);
    }
  }

  return s;
}

string triggerOutputEventToStr(const uint64_t triggerEvents)
{
  if(triggerEvents == TOE_UNKNOWN)
    return "Unknown";

  string s;

  for(unsigned int i = 0; i < TOEN_COUNT; i++)
  {
    if(triggerEvents & (1ULL << i))
    {
      if(s.length() != 0)
        s.append(", ");

      s.append(TriggerOutputEvents[i]);
    }
  }

  return s;
}

string boolToStr(const bool value)
{
  return value ? "true" : "false";
}
